# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open
from os import path

# The directory containing this file
HERE = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# This call to setup() does all the work
setup(
    name="csgapi",
    version="0.1.2",
    description="API for interacting with the Computer Systems Genome database.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://csgenome.org/",
    author="Ani Ramadoss",
    author_email="aniramadoss2002@vt.edu",
    license="MIT",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Operating System :: OS Independent"
    ],
    packages=["clientAPI", "clientAPI.objects"],
    py_modules=["clientAPI.base", "clientAPI.client", "clientAPI.mixin", "clientAPI.utils"],
    include_package_data=True,
    install_requires=["flask", "requests"]
)



from typing import cast
from ..base import RESTManager, RESTObject, RequiredOptional
from ..mixin import GetMixin, CreateMixin, UpdateMixin, SaveMixin, ListMixin

__all__ = [
    "SystemConfiguration",
    "SystemConfigurationManager",
]


class SystemConfiguration(SaveMixin, RESTObject):
    _id_attr = "uid"
    # pass


# NOTE: even tho Delete Mixin is integrated, since system_configuration is a look up table,
# our API does not support DELETE for it
class SystemConfigurationManager(GetMixin, CreateMixin, UpdateMixin, ListMixin, RESTManager):
    _path = "system_configurations"
    _obj_cls = SystemConfiguration
    # _create_attrs = RequiredOptional(required=("processor_configuration_id", "memory_configuration_id",
    #                                            "accelerator_configuration_id", "interconnect_configuration_id", "operating_system_configuration_id"))
    # TODO: figure out if the auth section should be passed in here,

    def get(self, uid=None, params=None):
        # NOTE: the casting allows each object to follow the logic implemented in the
        # MIXIN utils w/o the need to repeating it each time. it also allows customization
        # in each obj
        return cast(SystemConfiguration, super(SystemConfigurationManager, self).get(uid=uid, params=params))



from typing import cast
from ..base import RESTManager, RESTObject, RequiredOptional
from ..mixin import GetMixin, CreateMixin, UpdateMixin, SaveMixin, ListMixin
from ..utils import handle_client_exception


__all__ = [
    "OperatingSystem",
    "OperatingSystemManager",
]


class OperatingSystem(SaveMixin, RESTObject):
    _id_attr = "uid"
    # pass


# NOTE: even tho Delete Mixin is integrated, since operating_system is a look up table,
# our API does not support DELETE for it
class OperatingSystemManager(GetMixin, CreateMixin, UpdateMixin, ListMixin, RESTManager):
    _path = "operating_system"
    #TODO: find path name

    _obj_cls = OperatingSystem
    _create_attrs = RequiredOptional(required=("name",)) #"os_name", "kernel", "file_system", "os_family", "release_date", "end_of_life_date", "os_version" )) 
    #NOTE: Check if attributes are required or optional (for both create and update)
    #TODO: figure out if the auth section should be passed in here, 
    # if so we need the func in mixin to refactor attrs before calling POST
    _update_attrs = RequiredOptional(required=("name",)) #, optional=("kernel", "file_system", "os_family", "release_date", "end_of_life_date", "os_version"))
    #TODO: Fix attributes, which are required/optional? Why the discrepancy btwn
    # models.py and the schema?

    def get(self, uid=None, params=None):
        # NOTE: the casting allows each object to follow the logic implemented in the 
        # MIXIN utils w/o the need to repeating it each time. it also allows customization
        # in each obj
        return cast(OperatingSystem, super(OperatingSystemManager, self).get(uid=uid, params=params))


